@@ -9,22 +9,23 @@ package Vote::Count;
 use namespace::autoclean;
 use Moose;
 
-# use Data::Dumper;
+use Data::Dumper;
 use Time::Piece;
 use Path::Tiny;
 use Vote::Count::Matrix;
-use Storable 3.15 'dclone';
+use Vote::Count::ReadBallots qw( read_ballots read_range_ballots);
+# use Storable 3.15 'dclone';
 
 no warnings 'experimental';
 
-our $VERSION='1.05';
+our $VERSION='1.10';
 
 =head1 NAME
 
 Vote::Count
 
 
-=head1 VERSION 1.05
+=head1 VERSION 1.10
 
 =cut
 
@@ -32,44 +33,16 @@ Vote::Count
 
 has 'BallotSet' => ( is => 'ro', isa => 'HashRef', required => 1 );
 
-has 'Active' => (
-  is      => 'rw',
-  isa     => 'HashRef',
-  lazy    => 1,
-  builder => 'ResetActive',
-);
-
-sub ResetActive ( $self ) { return dclone $self->BallotSet()->{'choices'} }
-
-sub SetActive ( $self, $active ) {
-  # Force deref
-  $self->{'Active'} = dclone $active;
+sub _load_ballotset ( $self ) {
+  if ( $self->{'BallotSet'}{'read_ballots'} ) {
+    $self->{'BallotSet'}
+      = read_ballots( $self->{'BallotSet'}{'read_ballots'} );
+  } elsif ( $self->{'BallotSet'}{'read_range_ballots'} ) {
+    $self->{'BallotSet'}
+      = read_range_ballots( $self->{'BallotSet'}{'read_range_ballots'} );
+  }
 }
 
-# I was typing the equivalent too often. made a method.
-sub SetActiveFromArrayRef ( $self, $active ) {
-  $self->{'Active'} = { map { $_ => 1 } $active->@* };
-}
-
-sub GetActive ( $self ) {
-  # Force deref
-  my $active = $self->Active();
-  return dclone $active;
-}
-
-has TieBreakMethod => (
-  is       => 'rw',
-  isa      => 'Str',
-  required => 0,
-);
-
-# This is only used for the precedence tiebreaker!
-has PrecedenceFile => (
-  is       => 'rw',
-  isa      => 'Str',
-  required => 0,
-);
-
 has 'PairMatrix' => (
   is      => 'ro',
   isa     => 'Object',
@@ -90,16 +63,10 @@ sub _buildmatrix ( $self ) {
   );
 }
 
-sub UpdatePairMatrix ( $self, $active = undef ) {
-  $active = $self->Active() unless defined $active;
-  $self->{'PairMatrix'} = Vote::Count::Matrix->new(
-    BallotSet => $self->BallotSet(),
-    Active    => $active
-  );
-}
-
 sub BUILD {
   my $self = shift;
+  # If files were given to ballotset they need to be loaded
+  $self->_load_ballotset();
   # Verbose Log
   $self->{'LogV'} = localtime->cdate . "\n";
   # Debugging Log
@@ -110,7 +77,8 @@ sub BUILD {
 }
 
 # load the roles providing the underlying ops.
-with 
+with
+  'Vote::Count::Common',
   'Vote::Count::Approval',
   'Vote::Count::Borda',
   'Vote::Count::Floor',
@@ -121,41 +89,6 @@ with
   'Vote::Count::TopCount',
   ;
 
-sub VotesCast ( $self ) {
-  return $self->BallotSet()->{'votescast'};
-}
-
-sub VotesActive ( $self ) {
-  unless ( $self->BallotSet()->{'options'}{'rcv'} ) {
-    die "VotesActive Method only supports rcv"
-  }
-  my $set         = $self->BallotSet();
-  my $active      = $self->Active();
-  my $activeCount = 0;
-LOOPVOTESACTIVE:
-    for my $B ( values $set->{ballots}->%* ) {
-        for my $V ( $B->{'votes'}->@* ) {
-            if ( defined $active->{$V} ) {
-                $activeCount += $B->{'count'};
-                next LOOPVOTESACTIVE;
-            }
-        }
-    }
-  return $activeCount;
-}
-
-sub BallotSetType ( $self ) {
-  if ( $self->BallotSet()->{'options'}{'rcv'} ) {
-    return 'rcv';
-  }
-  elsif ( $self->BallotSet()->{'options'}{'range'} ) {
-    return 'range';
-  }
-  else {
-    die "BallotSetType is undefined or unknown type.";
-  }
-}
-
 __PACKAGE__->meta->make_immutable;
 1;
 
@@ -170,6 +103,9 @@ __PACKAGE__->meta->make_immutable;
 
 Provides a Toolkit for implementing multiple voting systems, allowing a wide range of method options. This library allows the creation of election resolution methods matching a set of Election Rules that are written in an organization's governing rules, and not requiring the bylaws to specify the rules of the software that will be used for the election, especially important given that many of the other libraries available do not provide a bylaws compatible explanation of their process.
 
+
+=head1 Synopsis
+
   use 5.022; # Minimum Perl, or any later Perl.
   use feature qw /postderef signatures/;
 
@@ -189,7 +125,7 @@ Provides a Toolkit for implementing multiple voting systems, allowing a wide ran
   # ApprovalFloor which defaulted to 5%.
   my $ChoicesAfterFloor = $CondorcetElection->ApprovalFloor();
   # Apply the ChoicesAfterFloor to the Election.
-  $CondorcetElection->Active( $ChoicesAfterFloor );
+  $CondorcetElection->SetActive( $ChoicesAfterFloor );
   # Get Smith Set and the Election with it as the Active List.
   my $SmithSet = $CondorcetElection->Matrix()->SmithSet() ;
   $CondorcetElection->logt(
@@ -203,7 +139,6 @@ Provides a Toolkit for implementing multiple voting systems, allowing a wide ran
     'Active' => $ChoicesAfterFloor );
   # Get a RankCount Object for the
   my $Plurality = $IRVElection->TopCount();
-  # In case of ties RankCount objects return top as an array, log the result.
   my $PluralityWinner = $Plurality->Leader();
   $IRVElection->logv( "Plurality Results", $Plurality->RankTable);
   if ( $PluralityWinner->{'winner'}) {
@@ -415,7 +350,7 @@ The objective is to have a library that can handle any of the myriad variants th
 
 =head2 Reading Ballots
 
-The Vote::Count::ReadBallots library provides functionality for reading files from disc. Currently it defines a format for a ballot file and reads that from disk. In the future additional formats may be added. Range Ballots may be in either JSON or YAML formats.
+The L<Vote::Count::ReadBallots|https://metacpan.org/pod/Vote::Count::ReadBallots> library provides functionality for reading files from disc. Currently it defines a format for a ballot file and reads that from disk. In the future additional formats may be added. Range Ballots may be in either JSON or YAML formats.
 
 
 =head2 Voting Method and Component Modules
@@ -460,76 +395,77 @@ When logging from your methods, use logt for events that produce a summary, use
 
 =head3 Active Sets
 
-Active sets are typically represented as a Hash Reference where the keys represent the active choices and the value is true. The VoteCount Object contains an Active Set which can be Accessed or set via the ->Active() method. The ->GetActive and ->SetActive methods are preferred because they break the reference link between the object's copy and the external copy of the Active set.
+Active sets are typically represented as a Hash Reference where the keys represent the active choices and the value is true. The VoteCount Object contains an Active Set which can be Accessed via the Active() method which will return a reference to the Active Set (changing the reference will change the active set). The GetActive and SetActive methods do not preserve any reference links and should be preferred. GetActiveList returns the Active Set as a sorted list.
 
-Most Components will take an argument for $activeset or default to the current Active set of the Vote::Count object, which will default to the Choices defined in the BallotSet.
+Many Components will take an argument for $activeset or default to the current Active set of the Vote::Count object, which will default to the Choices defined in the BallotSet.
 
 
 =head1 Vote::Count Methods
 
-=over
-
-=item *
-
-new
+Most of these are provided by the Role Common and available directly in both Matrix objects and Vote::Count Objects. Vote::Count objects create a child Matrix object: PairMatrix.
 
 
-
-=item *
-
-Active: Set or Get Active Set as HashRef
+=head3 new
 
 
+=head3 Active
 
-=item *
-
-ResetActive: Sets the Active Set to the full choices list of the BallotSet.
+Get Active Set as HashRef to the active set. Changing the new HashRef will change the internal Active Set, GetActive is recommended as it will return a HashRef that is a copy instead.
 
 
+=head3 GetActive
 
-=item *
-
-SetActive: Sets the Active Set to provided HashRef. Using the Active method may preserve a reference between the Active Set and the HashRef, SetActive will not. The values to the hashref should evaluate as True.
+Returns a hashref containing a copy of the Active Set.
 
 
+=head3 GetActiveList
 
-=item *
-
-SetActiveFromArrayRef: Same as SetActive except it takes an ArrayRef of the choices to be set as Active.
+Returns a simple array of the members of the Active Set.
 
 
+=head3 ResetActive
 
-=item *
-
-BallotSet: Get BallotSet
+Sets the Active Set to the full choices list of the BallotSet.
 
 
+=head3 SetActive
 
-=item *
-
-PairMatrix: Get a Matrix Object for the Active Set. Generated and cached on the first request.
+Sets the Active Set to provided HashRef. The values to the hashref should evaluate as True.
 
 
+=head3 SetActiveFromArrayRef
 
-=item *
-
-UpdatePairMatrix: Regenerate and cache Matrix with current Active Set.
+Same as SetActive except it takes an ArrayRef of the choices to be set as Active.
 
 
+=head3 BallotSet
 
-=item *
-
-VotesCast: Returns the number of votes cast.
+Get BallotSet
 
 
+=head3 PairMatrix
 
-=item *
-
-VotesActive: Returns the number of non-exhausted ballots based on the current Active Set.
+Get a Matrix Object for the Active Set. Generated and cached on the first request.
 
 
+=head3 UpdatePairMatrix
 
-=back
+Regenerate and cache Matrix with current Active Set.
+
+
+=head3 VotesCast
+
+Returns the number of votes cast.
+
+
+=head3 VotesActive
+
+Returns the number of non-exhausted ballots based on the current Active Set.
+
+
+=head1 Minimum Perl Version
+
+It is the policy of Vote::Count to only develop with recent versions of Perl. Support for older versions will be dropped as they either start failing tests or impair adoption of new features.
 
 
 =head2 Components
@@ -543,7 +479,7 @@ Directory of Vote Counting Methods linking to the Vote::Count module for it.
 
 =item *
 
-L<Catalog|https://metacpan.org/pod/distribution/Vote-Count/Catalog.pod>
+L<Catalog|https://metacpan.org/pod/distribution/Vote-Count/lib/Vote/Catalog.pod>
 
 
 =back
@@ -632,6 +568,12 @@ L<Vote::Count::Method::CondorcetIRV|https://metacpan.org/pod/Vote::Count::Method
 L<Vote::Count::Method::CondorcetVsIRV|https://metacpan.org/pod/Vote::Count::Method::CondorcetVsIRV>
 
 
+=item *
+
+L<Vote::Count::Method::MinMax|https://metacpan.org/pod/Vote::Count::Method::MinMax>
+
+
+
 =item *
 
 L<Vote::Count::Method::STAR|https://metacpan.org/pod/Vote::Count::Method::STAR>
@@ -700,36 +642,6 @@ L<Vote::Count::Start|https://metacpan.org/pod/Vote::Count::Start>
 
 This project needs contributions from Programmers and Mathematicians. Review and citations from Mathematicians are urgently requested, because in addition to being a Tool-set for implementing vote counting this documentation will for many also be the manual. From coders there is a lot of help that could be given: any well known method could use a write up if it is easy to implement with the toolkit (see Benham) or a code submission if it is not. Currently Tiedeman, SSD, and Kemmeny-Young are unimplemented.
 
-
-=head1 Advice, Recommendations, Opinion
-
-This section is highly opinionated by the Author of Vote::Count.
-
-If you're looking at all of this wondering "which method I should recommend to my organization to implement Ranked Choice Voting internally?" this is the advice offered by the author of Vote::Count.
-
-I<Instant Runoff Voting> is simple, easy to count by hand, Later Harm protected, and is the most widely used method. It has serious consistency issues, especially how poorly it handles common cloning situations.
-
-I<Benham Condorcet IRV>, meets the two main Condorcet Criteria, and is countable by hand, but it fails Later Harm.
-
-Benham and IRV are good choices for Hand Count Methods.
-
-I<Smith Set IRV> meets all three key criteria for Condorcet Methods and has less Later Harm effect than any other (non-redacting) Condorcet Method. It is simple to understand, but not practical for hand counting.
-
-If you like I<Borda> or prefer a I<Range Ballot>, my pick is for I<STAR>.
-
-STAR is handcountable but requires a Range Ballot. Range methods like STAR have less Later Harm effect than Borda Methods.
-
-I<Redacting Condorcet Methods> are the B<best> for a conventional Ranked Choice Ballot. If a Condorcet Winner does not create a Later Harm violation they will always be chosen. They can create a gauge of the later harm effect that then allows for the establishment of a Later Harm tolerance. The steps for I<Condorcet Vs IRV> are easy to understand but the number of steps qualifies it as somewhat complex. Other methods in the family (not yet implemented) will be more complex.
-
-B<Redacting Condorcet> (Condorcet Vs IRV being the only one available here at the moment) is my preference. B<STAR> is my preferred Scoring method. If you need to hand count, Benham is your Condorcet Method and IRV is your Later Harm Protected Method. Smith Set IRV is a simple Condorcet Method that is better on Later Harm than any other non-redacting Condorcer Method, it is a much better choice than Benham. If Later Harm compliance is required a Redacting Condorcet Method is your best choice, and IRV your choice if they're too complex.
-
-
-=head2 Floor Rules and Tie Breakers
-
-In real world elections it is typical to have a number of choices that recieve very little support. A Floor Rule allows quick elimination of these choices, but don't help when voters rank all choices. STAR is an exception and does not benefit from a Floor Rule. 5% Approval is a good weak Floor, and TCA is a good aggressive one.
-
-Ties are inescapable. Modified Grand Junction has the maximum resolvability, but has a Later Harm effect. For a Later Harm safe Tie Breaker Eliminate All is effective (except at the final step).
-
 =cut
 
 #buildpod
